syntax = "proto3";

package aegis;

import "cell.proto";
import "spawn.proto";

message World {
  int32 width = 1;
  int32 height = 2;
  int32 seed = 3;
  int32 start_energy = 4;
  repeated Cell cells = 5;
  int32 total_survivors = 6;
  repeated InitSpawn init_spawns = 7;
}
