# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: team.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'team.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nteam.proto\x12\x05\x61\x65gis\"\xc0\x01\n\x08TeamInfo\x12\x13\n\x0bsaved_alive\x18\x01 \x01(\x05\x12\x12\n\nsaved_dead\x18\x02 \x01(\x05\x12\r\n\x05saved\x18\x03 \x01(\x05\x12\x17\n\x0fpredicted_right\x18\x04 \x01(\x05\x12\x17\n\x0fpredicted_wrong\x18\x05 \x01(\x05\x12\x11\n\tpredicted\x18\x06 \x01(\x05\x12\r\n\x05score\x18\x07 \x01(\x05\x12\r\n\x05units\x18\x08 \x01(\x05\x12\x19\n\x04team\x18\t \x01(\x0e\x32\x0b.aegis.Team* \n\x04Team\x12\t\n\x05GOOBS\x10\x00\x12\r\n\tVOIDSEERS\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'team_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_TEAM']._serialized_start=216
  _globals['_TEAM']._serialized_end=248
  _globals['_TEAMINFO']._serialized_start=22
  _globals['_TEAMINFO']._serialized_end=214
# @@protoc_insertion_point(module_scope)
